@@ -1,9 +1,9 @@
 import pkginfo
+import re
 import shutil
 import tempfile
 import json
 from collections import defaultdict
-from django.core.files.storage import default_storage as storage
 from django.conf import settings
 from jinja2 import Template
 from packaging.utils import canonicalize_name
@@ -52,6 +52,20 @@ DIST_EXTENSIONS = {
     ".zip": "sdist",
 }
 
+DIST_REGEXES = {
+    # regex from https://github.com/pypa/pip/blob/18.0/src/pip/_internal/wheel.py#L569
+    ".whl": re.compile(
+        r"""^(?P<name>.+?)-(?P<version>.*?)
+        ((-(?P<build>\d[^-]*?))?-(?P<pyver>.+?)-(?P<abi>.+?)-(?P<plat>.+?)
+        \.whl|\.dist-info)$""",
+        re.VERBOSE
+    ),
+    # regex based on https://setuptools.pypa.io/en/latest/deprecated/python_eggs.html#filename-embedded-metadata  # noqa: E501
+    ".egg": re.compile(r"^(?P<name>.+?)-(?P<version>.*?)(-(?P<pyver>.+?(-(?P<plat>.+?))?))?\.egg|\.egg-info$"),  # noqa: E501
+    # regex based on https://github.com/python/cpython/blob/v3.7.0/Lib/distutils/command/bdist_wininst.py#L292  # noqa: E501
+    ".exe": re.compile(r"^(?P<name>.+?)-(?P<version>.*?)\.(?P<plat>.+?)(-(?P<pyver>.+?))?\.exe$"),
+}
+
 DIST_TYPES = {
     "bdist_wheel": pkginfo.Wheel,
     "bdist_wininst": pkginfo.Distribution,
@@ -73,6 +87,8 @@ def parse_project_metadata(project):
     """
     package = {}
     package['name'] = project.get('name') or ""
+    package['version'] = project.get('version') or ""
+    package['packagetype'] = project.get('packagetype') or ""
     package['metadata_version'] = project.get('metadata_version') or ""
     package['summary'] = project.get('summary') or ""
     package['description'] = project.get('description') or ""
@@ -87,6 +103,7 @@ def parse_project_metadata(project):
     package['project_url'] = project.get('project_url') or ""
     package['platform'] = project.get('platform') or ""
     package['supported_platform'] = project.get('supported_platform') or ""
+    package['requires_python'] = project.get('requires_python') or ""
     package['requires_dist'] = json.dumps(project.get('requires_dist', []))
     package['provides_dist'] = json.dumps(project.get('provides_dist', []))
     package['obsoletes_dist'] = json.dumps(project.get('obsoletes_dist', []))
@@ -94,6 +111,7 @@ def parse_project_metadata(project):
     package['classifiers'] = json.dumps(project.get('classifiers', []))
     package['project_urls'] = json.dumps(project.get('project_urls', {}))
     package['description_content_type'] = project.get('description_content_type') or ""
+    package['python_version'] = project.get('python_version') or ""
 
     return package
 
@@ -114,17 +132,15 @@ def parse_metadata(project, version, distribution):
         dictionary: of useful python metadata
 
     """
-    package = {}
+    package = parse_project_metadata(project)
 
     package['filename'] = distribution.get('filename') or ""
     package['packagetype'] = distribution.get('packagetype') or ""
     package['version'] = version
     package['url'] = distribution.get('url') or ""
     package['sha256'] = distribution.get('digests', {}).get('sha256') or ""
-    package['python_version'] = distribution.get('python_version') or ""
-    package['requires_python'] = distribution.get('requires_python') or ""
-
-    package.update(parse_project_metadata(project))
+    package['python_version'] = distribution.get('python_version') or package.get('python_version')
+    package['requires_python'] = distribution.get('requires_python') or package.get('requires_python')  # noqa: E501
 
     return package
 
@@ -144,15 +160,35 @@ def get_project_metadata_from_artifact(filename, artifact):
     # because pkginfo validates that the filename has a valid extension before
     # reading it
     with tempfile.NamedTemporaryFile('wb', dir=".", suffix=filename) as temp_file:
-        artifact_file = storage.open(artifact.file.name)
-        shutil.copyfileobj(artifact_file, temp_file)
+        shutil.copyfileobj(artifact.file, temp_file)
         temp_file.flush()
         metadata = DIST_TYPES[packagetype](temp_file.name)
         metadata.packagetype = packagetype
+        if packagetype == "sdist":
+            metadata.python_version = "source"
+        else:
+            pyver = ""
+            regex = DIST_REGEXES[extensions[pkg_type_index]]
+            if bdist_name := regex.match(filename):
+                pyver = bdist_name.group("pyver") or ""
+            metadata.python_version = pyver
         return metadata
 
 
-def python_content_to_json(base_path, content_query, version=None):
+def artifact_to_python_content_data(filename, artifact, domain=None):
+    """
+    Takes the artifact/filename and returns the metadata needed to create a PythonPackageContent.
+    """
+    metadata = get_project_metadata_from_artifact(filename, artifact)
+    data = parse_project_metadata(vars(metadata))
+    data['sha256'] = artifact.sha256
+    data['filename'] = filename
+    data['pulp_domain'] = domain or artifact.pulp_domain
+    data['_pulp_domain'] = data['pulp_domain']
+    return data
+
+
+def python_content_to_json(base_path, content_query, version=None, domain=None):
     """
     Converts a QuerySet of PythonPackageContent into the PyPi JSON format
     https://www.python.org/dev/peps/pep-0566/
@@ -169,8 +205,8 @@ def python_content_to_json(base_path, content_query, version=None):
     if not latest_content:
         return None
     full_metadata.update({"info": python_content_to_info(latest_content[0])})
-    full_metadata.update({"releases": python_content_to_releases(content_query, base_path)})
-    full_metadata.update({"urls": python_content_to_urls(latest_content, base_path)})
+    full_metadata.update({"releases": python_content_to_releases(content_query, base_path, domain)})
+    full_metadata.update({"urls": python_content_to_urls(latest_content, base_path, domain)})
     return full_metadata
 
 
@@ -245,25 +281,27 @@ def python_content_to_info(content):
     }
 
 
-def python_content_to_releases(content_query, base_path):
+def python_content_to_releases(content_query, base_path, domain=None):
     """
     Takes a QuerySet of PythonPackageContent and returns a dictionary of releases
     with each key being a version and value being a list of content for that version of the package
     """
     releases = defaultdict(lambda: [])
     for content in content_query:
-        releases[content.version].append(python_content_to_download_info(content, base_path))
+        releases[content.version].append(
+            python_content_to_download_info(content, base_path, domain)
+        )
     return releases
 
 
-def python_content_to_urls(contents, base_path):
+def python_content_to_urls(contents, base_path, domain=None):
     """
     Takes the latest content in contents and returns a list of download information
     """
-    return [python_content_to_download_info(content, base_path) for content in contents]
+    return [python_content_to_download_info(content, base_path, domain) for content in contents]
 
 
-def python_content_to_download_info(content, base_path):
+def python_content_to_download_info(content, base_path, domain=None):
     """
     Takes in a PythonPackageContent and base path of the distribution to create a dictionary of
     download information for that content. This dictionary is used by Releases and Urls.
@@ -280,20 +318,25 @@ def python_content_to_download_info(content, base_path):
     origin = settings.CONTENT_ORIGIN.strip("/")
     prefix = settings.CONTENT_PATH_PREFIX.strip("/")
     base_path = base_path.strip("/")
-    url = "/".join((origin, prefix, base_path, content.filename))
+    components = [origin, prefix, base_path, content.filename]
+    if domain:
+        components.insert(2, domain.name)
+    url = "/".join(components)
+    md5 = artifact.md5 if artifact and artifact.md5 else ""
+    size = artifact.size if artifact and artifact.size else 0
     return {
         "comment_text": "",
-        "digests": {"md5": artifact.md5, "sha256": artifact.sha256},
+        "digests": {"md5": md5, "sha256": content.sha256},
         "downloads": -1,
         "filename": content.filename,
         "has_sig": False,
-        "md5_digest": artifact.md5,
+        "md5_digest": md5,
         "packagetype": content.packagetype,
         "python_version": content.python_version,
         "requires_python": content.requires_python or None,
-        "size": artifact.size,
-        "upload_time": str(artifact.pulp_created),
-        "upload_time_iso_8601": str(artifact.pulp_created.isoformat()),
+        "size": size,
+        "upload_time": str(content.pulp_created),
+        "upload_time_iso_8601": str(content.pulp_created.isoformat()),
         "url": url,
         "yanked": False,
         "yanked_reason": None
